@@ -15,6 +15,15 @@ def fix_keywords(value: str) -> str:
     return value
 
 
+RESERVED_WORDS = ("self",)
+
+
+def fix_reserved_words(value: str) -> str:
+    if value in RESERVED_WORDS:
+        return f"{value}_"
+    return value
+
+
 def group_title(value: str) -> str:
     value = re.sub(r"([A-Z]{2,})([A-Z][a-z]|[ \-_]|$)", lambda m: m.group(1).title() + m.group(2), value.strip())
     value = re.sub(r"(^|[ _-])([A-Z])", lambda m: m.group(1) + m.group(2).lower(), value)
@@ -49,7 +58,7 @@ def to_valid_python_identifier(value: str) -> str:
     See:
         https://docs.python.org/3/reference/lexical_analysis.html#identifiers
     """
-    new_value = fix_keywords(sanitize(value))
+    new_value = fix_reserved_words(fix_keywords(sanitize(value)))
 
     if new_value.isidentifier():
         return new_value
